#include "mcr_gpt_lib.h"
#include <cmath>
void countup(void);
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args);

//制御対象(CN1)
#define LEDR      13
#define LEDB      12
#define LEDG      11
#define BZ        10
#define LED1      9
#define LED2      8
#define CK        7

//制御対象(CN2)
#define LEDa      61
#define LEDb      62
#define LEDc      63
#define LEDd      64
#define LEDe      65
#define LEDf      66
#define LEDg      67
#define LEDdp     68

//設計製作
#define RANSE     A0
#define ENC       3
#define TS1       6
#define TS2       5
#define TGS       4

// グローバル変数の宣言
FspTimer fsp_timer;       // 割り込み関係

void config_init(){
//ピンモード設定
//制御対象(CN1)
pinMode(LEDR, OUTPUT);
pinMode(LEDB, OUTPUT);
pinMode(LEDG, OUTPUT);
pinMode(BZ, OUTPUT);
pinMode(LED1, OUTPUT);
pinMode(LED2, OUTPUT);
pinMode(CK, OUTPUT);

//制御対象(CN2)
pinMode(LEDa, OUTPUT);
pinMode(LEDb, OUTPUT);
pinMode(LEDc, OUTPUT);
pinMode(LEDd, OUTPUT);
pinMode(LEDe, OUTPUT);
pinMode(LEDf, OUTPUT);
pinMode(LEDg, OUTPUT);
pinMode(LEDdp, OUTPUT);

//設計製作
pinMode(TS1, INPUT);
pinMode(TS2, INPUT);
pinMode(TGS, INPUT);
pinMode(RANSE, INPUT);

attachInterrupt(digitalPinToInterrupt(ENC),countup,CHANGE);

// AGT 1msごとの割り込み処理の設定 PCLKB=24MHz ∴TIMER_SOURCE_DIV_1(1分周)なら、1/(24e6*1) * 24000 = 1ms  設定は１小さい値である23999を設定する
  fsp_timer.begin(TIMER_MODE_PERIODIC, AGT_TIMER, 1, 23999, 1, (timer_source_div_t)TIMER_SOURCE_DIV_1, AGTCallback);
  IRQManager::getInstance().addPeripheral(IRQ_AGT, (void*)fsp_timer.get_cfg());
  fsp_timer.open(); 

}

//関数宣言
volatile int ENCcount,ranbase_count,enc_base;
volatile unsigned int cnt = 0;
volatile char left_seg,right_seg;
volatile unsigned int stm_cnt = 0;
volatile unsigned int dc_dir,dc_sp,fled,stm_no,base_count;
volatile int stm_count;
volatile float ran_count,ran_Vcc,enc_sub,enc_count;
volatile byte stm_sp,stm_dir;

//7セグ表示
const byte num[] = {		 //A←表示される値(11)←プログラムに入力する数値
  0b11111100, // 0   (0)
  0b01100000, // 1   (1)
  0b11011010, // 2   (2)
  0b11110010, // 3   (3)
  0b01100110, // 4   (4)
  0b10110110, // 5   (5)
  0b10111110, // 6   (6)
  0b11100100, // 7   (7)
  0b11111110, // 8   (8)
  0b11110110, // 9   (9)
  0b11101110, // A   (10)
  0b00111110, // b   (11)
  0b00011010, // c   (12)
  0b01111010, // d   (13)
  0b10011110, // E   (14)
  0b10001110, // F   (15)
  0b00000001, // .   (16)
  0b00000000, // 空白(17)
  0b10011100, //  [　(18)
  0b11110000, //  ]　(19)
  0b00000010, //  -　(20)
  //豆知識；実は0~15の数字、文字順は16進数と一致する
};

//FLED
#define RED  0x04;
#define BLUE  0x02;
#define GREEN  0x01;
#define MAZENTA  0x06;
#define CYAN  0x03;
#define YELLOW  0x05;
#define WHITE  0x07;
#define BLACK  0x00;

//MOTOR
#define TOKEI     0x01
#define HANTOKEI  0x02

//ステッピングモーター動作パルス
const byte stm_rot[] = {
  0b00000001,
  0b00000011,
  0b00000010,
  0b00000110,
  0b00000100,
  0b00001100,
  0b00001000,
  0b00001001
};

void countup(){

  //正転でカウントアップ・逆転でカウントダウン(どちらかのプログラムはコメントアウトしておく)
  if(dc_dir == TOKEI){
    ENCcount++;
  }else{
    ENCcount--;
  }

  //正転・逆転でカウントアップ(どちらかのプログラムはコメントアウトしておく)
  /*
  ENCcount++;
  */

}

//割り込み関数　約1ms
void AGTCallback(timer_callback_args_t __attribute((unused)) * p_args) {
  cnt++;

  //フォトインタラプタのカウント
  enc_base = ENCcount / 8;
  enc_sub = ENCcount % 8;
  enc_sub = enc_sub / 8;
  enc_count = enc_base + enc_sub;

  digitalWrite(LEDR, (fled>>2)&0x01);
  digitalWrite(LEDB, (fled>>1)&0x01);
  digitalWrite(LEDG, fled&0x01);

  //スライダーの距離計算
  if(cnt%100==0){                                     //100msに一回動作する(詳細版)
    ranbase_count = base_count / 100;                 //100回の計測の平均値を出す(詳細版)
    base_count = 0;                                   //リセットして次の計測に備える(詳細版)
    ranbase_count = map(ranbase_count,6,1068,0,500);  //読込んだ値から電圧値に変換する(0-5V)(詳細版)
    ran_Vcc = ranbase_count;                          //ズレ修正用(詳細版)
    ran_Vcc = ran_Vcc / 100;                          //ズレ修正用(詳細版)
    ranbase_count = ranbase_count - 225;              //数値の初期値が0になるように最低電圧値×100の値を引く(電圧はSerial.printnの計算電圧値を使って良い)(詳細版)
    if(ranbase_count>0){                              //0を下回った時に0に固定(詳細版)
      ranbase_count = 0;
    }
    ranbase_count = std::fabs(ranbase_count);         //絶対値化(詳細版)
    ran_count = ranbase_count * 0.275 + 4;            //センチに返還(詳細版)
  }else{
    base_count = base_count + analogRead(RANSE);      //100回分の計測値を足す(詳細版)
  }
  
  //ran_count = map(analogRead(RANSE),6,1068,0,500);  //読込んだ値から電圧値に変換する(簡易版)
  //ran_count = map(ran_count,27,231,58,4);           //電圧の値からセンチに変換する(簡易版)

  //詳細版：毎秒10回100回の測定データを元にミリレベルで数値を出力する。
  //簡易版：毎ミリ秒ごとにセンチレベルで数値を出力する。
  //追記：もしかしたら簡易版のほうが安定するかもしれないが、細かい値がほしければ詳細版一択だろう。
  //また詳細版の各計算に用いている数値は独自に導き出したものであり、パーツが変わった際の互換性は不明。

  //７セグ・DCモータ・ステッピングモータ各割当
  if( cnt % 3 == 0 ){                                 //7セグ左右逆問題修正済み
    digitalWrite(LED1, 1);
    digitalWrite(LED2, 0);
    digitalWrite(LEDa, (right_seg>>7)&0x01);
    digitalWrite(LEDb, (right_seg>>6)&0x01);
    digitalWrite(LEDc, (right_seg>>5)&0x01);
    digitalWrite(LEDd, (right_seg>>4)&0x01);
    digitalWrite(LEDe, (right_seg>>3)&0x01);
    digitalWrite(LEDf, (right_seg>>2)&0x01);
    digitalWrite(LEDg, (right_seg>>1)&0x01 );
    digitalWrite(LEDdp, right_seg&0x01);
  }else if( cnt % 3 == 1 ){                           //7セグ左右逆問題修正済み
    digitalWrite(LED1, 0);
    digitalWrite(LED2, 1);
    digitalWrite(LEDa, (left_seg>>7)&0x01);
    digitalWrite(LEDb, (left_seg>>6)&0x01);
    digitalWrite(LEDc, (left_seg>>5)&0x01);
    digitalWrite(LEDd, (left_seg>>4)&0x01);
    digitalWrite(LEDe, (left_seg>>3)&0x01);
    digitalWrite(LEDf, (left_seg>>2)&0x01);
    digitalWrite(LEDg, (left_seg>>1)&0x01 );
    digitalWrite(LEDdp, left_seg&0x01);
  }else if( cnt % 3 == 2 ){
    digitalWrite(LED1, 0);
    digitalWrite(LED2, 0);
    stm_cnt++;
    if(dc_sp != 0){
      if(stm_cnt%dc_sp==0){
        digitalWrite(LEDg, (dc_dir>>1)&0x01);
        digitalWrite(LEDdp, dc_dir&0x01);
      }else{
        digitalWrite(LEDg, 0);
        digitalWrite(LEDdp, 0);
      }
    }else{
      digitalWrite(LEDg, 0);
      digitalWrite(LEDdp, 0);
    }
    if(stm_sp != 0){   
      if(stm_cnt%stm_sp == 0){
          if(stm_dir == 0x02){
              stm_no++;
              stm_count--;
          }
          if(stm_dir == 0x01){
              stm_no--;
              stm_count++;
          }
      }
      digitalWrite(LEDa, stm_rot[stm_no%8]&0x01);
      digitalWrite(LEDb, (stm_rot[stm_no%8]>>1)&0x01);
      digitalWrite(LEDc, (stm_rot[stm_no%8]>>2)&0x01);
      digitalWrite(LEDd, (stm_rot[stm_no%8]>>3)&0x01);
    }else{
      digitalWrite(LEDa, 0);
      digitalWrite(LEDb, 0);
      digitalWrite(LEDc, 0);
      digitalWrite(LEDd, 0);
    }
    digitalWrite(CK, 1);
    digitalWrite(CK, 0);
  }
}



